// ===========================================================================
//	CCMWindowAttachment.cp	1998-2002 Metrowerks Corporation.  All rights reserved.
// ===========================================================================
//	Original Author: John C. Daub
//
//	A variant of LCMAttachment slightly modified for a window

#ifdef PowerPlant_PCH
	#include PowerPlant_PCH
#endif

#include "CCMWindowAttachment.h"
#include "CCloseTask.h"
#include <LMenu.h>

const CommandT	cmd_Beep		= FOUR_CHAR_CODE('BEEP');
const CommandT	cmd_BeepTwice	= FOUR_CHAR_CODE('BEP2');


// ---------------------------------------------------------------------------
//	 CCMWindowAttachment					Default Constructor		  [public]
// ---------------------------------------------------------------------------

CCMWindowAttachment::CCMWindowAttachment()
{
	mSubMenuH	= nil;
}


// ---------------------------------------------------------------------------
//	 CCMWindowAttachment					Stream Constructor		  [public]
// ---------------------------------------------------------------------------

CCMWindowAttachment::CCMWindowAttachment(
	LStream*	inStream)

	: LCMAttachment(inStream)
{
	mSubMenuH	= nil;
}


// ---------------------------------------------------------------------------
//	 ~CCMWindowAttachment					Destructor		 [public, virtual]
// ---------------------------------------------------------------------------

CCMWindowAttachment::~CCMWindowAttachment()
{
	DisposeSubMenu();
}


// ---------------------------------------------------------------------------
//	 DispatchCommand									[protected, virtual]
// ---------------------------------------------------------------------------
//	Overridden to special-case for cmd_Close. Issuing a cmd_Close from within
//	the Attachment will cause the Attachment to be indirectly deleted (by
//	virtue of closing the window, which deletes the window, which then
//	deletes all of the window's contents, including this Attachment instance).
//	This causes some potential pain as LCMAttachment and LAttachment cannot
//	know "this" has been deleted out from under itself. Since both
//	LCMAttachment and LAttachment must access data members and cannot know
//	if "this" remains valid, there is potential for the Attachment to
//	access invalid memory. This can cause complaints from checkers like
//	Spotlight, as well as crashes and other bad things.
//
//	To work around this, we use CCloseTask (derived from LOneTimeTask) to
//	defer the closing of the window. This will allow the stack to properly
//	unwind and place us in a "safe place" (back in the event loop), and
//	at DevoteTimeToRepeaters() time, the window will be deleted.

void
CCMWindowAttachment::DispatchCommand(
	UInt32		inSelectionType,
	CommandT	inCommand)
{
	if ((inSelectionType == kCMMenuItemSelected) && (inCommand == cmd_Close)) {
		LCommander*	target = FindCommandTarget();
		if (target != nil) {
			new CCloseTask(target);
		}
	} else {
		LCMAttachment::DispatchCommand(inSelectionType, inCommand);
	}
}


// ---------------------------------------------------------------------------
//	 PopulateMenu										[protected, virtual]
// ---------------------------------------------------------------------------

void
CCMWindowAttachment::PopulateMenu(
	MenuHandle	inMenuH)
{
	LCMAttachment::PopulateMenu(inMenuH);
	
		// If we have a "Close" item, let's rename it to
		// something a wee more descriptive
	
	for (SInt16 i = 1; i <= ::CountMenuItems(inMenuH); i++) {
		LStr255 menuText;
		::GetMenuItemText(inMenuH, i, menuText);
		if (::EqualString( menuText, StringLiteral_("Close"), false, false)) {
			menuText += StringLiteral_(" Window");
			::SetMenuItemText(inMenuH, i, menuText);
		}
	}
	
		// And to show a few other things:
	
		// Add a divider line
	AddCommand(inMenuH, cmd_Nothing);
	
		// A command not already in the menu bar
	AddCommand(inMenuH, cmd_Beep, StringLiteral_("Beep"));
	
		// Add a submenu
	DisposeSubMenu();
	
	SInt16		subID = 200;		// poor technique..... Something like
									// LDebugAttachment::FindUniqueMenuID needed
									// here to ensure no conflicts. But, this is
									// just a demo.
									// (BTW, remember that subMenu's can only have
									// ID's within a certain range)
									
	mSubMenuH = LMenu::MakeNewMacMenu(subID, Str_Empty, false);
	
	if (mSubMenuH) {

		AddCommand(inMenuH, cmd_Beep, StringLiteral_("Beep Submenu"));	

		AddCommand(mSubMenuH, cmd_Beep, StringLiteral_("Beep Once"));
		AddCommand(mSubMenuH, cmd_BeepTwice, StringLiteral_("Beep Twice"));
		
		::MacInsertMenu(mSubMenuH, hierMenu);
		::SetMenuItemHierarchicalID(inMenuH, ::CountMenuItems(inMenuH), subID);
	}
}


// ---------------------------------------------------------------------------
//	 DiposeSubMenu
// ---------------------------------------------------------------------------
//	A little utility for cleaning up after ourselves.

void
CCMWindowAttachment::DisposeSubMenu()
{
	if (mSubMenuH != nil) {
#if ACCESSOR_CALLS_ARE_FUNCTIONS
		::MacDeleteMenu(::GetMenuID(mSubMenuH));
#else
		::MacDeleteMenu((*mSubMenuH)->menuID);
#endif
		::DisposeMenu(mSubMenuH);
		mSubMenuH = nil;
	}
}